@@ -11,21 +11,24 @@ macro_rules! impl_deserialize_n {
         where
             V: serde::de::Visitor<'de>,
         {
-            visitor.$visit(
-                self.read
+            visitor.$visit({
+                let value = self
+                    .read
                     .get_attribute_value(&self.current_field)
                     .ok_or_else(|| Error {
                         message: format!("missing integer for field \"{}\"", self.current_field),
                     })?
-                    .clone()
+                    .clone();
+                value
                     .as_i64()
+                    .or_else(|| value.as_bytes().map(|v| v as i64))
                     .ok_or_else(|| Error {
                         message: format!(
                             "Invalid type for field \"{}\", expected integer",
                             self.current_field
                         ),
-                    })?,
-            )
+                    })?
+            })
         }
     };
     ($type:ty, $method:ident, $visit:ident) => {
@@ -33,21 +36,24 @@ macro_rules! impl_deserialize_n {
         where
             V: serde::de::Visitor<'de>,
         {
-            visitor.$visit(
-                self.read
+            visitor.$visit({
+                let value = self
+                    .read
                     .get_attribute_value(&self.current_field)
                     .ok_or_else(|| Error {
                         message: format!("missing integer for field \"{}\"", self.current_field),
                     })?
-                    .clone()
+                    .clone();
+                value
                     .as_i64()
+                    .or_else(|| value.as_bytes().map(|v| v as i64))
                     .ok_or_else(|| Error {
                         message: format!(
                             "Invalid type for field \"{}\", expected integer",
                             self.current_field
                         ),
-                    })? as $type,
-            )
+                    })? as $type
+            })
         }
     };
 }
@@ -57,21 +63,24 @@ macro_rules! impl_deserialize_f {
         where
             V: serde::de::Visitor<'de>,
         {
-            visitor.$visit(
-                self.read
+            visitor.$visit({
+                let value = self
+                    .read
                     .get_attribute_value(&self.current_field)
                     .ok_or_else(|| Error {
                         message: format!("missing float for field \"{}\"", self.current_field),
                     })?
-                    .clone()
+                    .clone();
+                value
                     .as_f64()
+                    .or_else(|| value.as_bytes().map(|v| v as f64))
                     .ok_or_else(|| Error {
                         message: format!(
                             "Invalid type for field \"{}\", expected float",
                             self.current_field
                         ),
-                    })?,
-            )
+                    })?
+            })
         }
     };
     ($type:ty, $method:ident, $visit:ident) => {
@@ -79,21 +88,24 @@ macro_rules! impl_deserialize_f {
         where
             V: serde::de::Visitor<'de>,
         {
-            visitor.$visit(
-                self.read
+            visitor.$visit({
+                let value = self
+                    .read
                     .get_attribute_value(&self.current_field)
                     .ok_or_else(|| Error {
                         message: format!("missing float for field \"{}\"", self.current_field),
                     })?
-                    .clone()
+                    .clone();
+                value
                     .as_f64()
+                    .or_else(|| value.as_bytes().map(|v| v as f64))
                     .ok_or_else(|| Error {
                         message: format!(
                             "Invalid type for field \"{}\", expected float",
                             self.current_field
                         ),
-                    })? as $type,
-            )
+                    })? as $type
+            })
         }
     };
 }
@@ -755,8 +767,10 @@ where
 }
 
 #[cfg(test)]
+#[allow(dead_code)]
 mod tests {
     use crate::Hocon;
+    use linked_hash_map::LinkedHashMap;
     use serde::Deserialize;
     use std::collections::HashMap;
 
@@ -777,24 +791,24 @@ mod tests {
 
     #[test]
     fn can_deserialize_struct() {
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("int"), Hocon::Integer(56));
         hm.insert(String::from("float"), Hocon::Real(543.12));
         hm.insert(String::from("boolean"), Hocon::Boolean(false));
         hm.insert(String::from("string"), Hocon::String(String::from("test")));
         let mut vec_sub = vec![];
-        let mut subhm = HashMap::new();
+        let mut subhm = LinkedHashMap::new();
         subhm.insert(String::from("int"), Hocon::Integer(5));
         subhm.insert(String::from("float"), Hocon::Integer(6));
         subhm.insert(String::from("extra"), Hocon::Integer(10));
         let subdoc = Hocon::Hash(subhm);
         vec_sub.push(subdoc);
-        let mut subhm = HashMap::new();
+        let mut subhm = LinkedHashMap::new();
         subhm.insert(String::from("int"), Hocon::Integer(5));
         subhm.insert(String::from("float"), Hocon::Integer(6));
         let subdoc = Hocon::Hash(subhm);
         vec_sub.push(subdoc);
-        let mut subhm = HashMap::new();
+        let mut subhm = LinkedHashMap::new();
         subhm.insert(String::from("int"), Hocon::Integer(5));
         subhm.insert(String::from("float"), Hocon::Integer(6));
         subhm.insert(String::from("extra"), Hocon::Null);
@@ -809,7 +823,7 @@ mod tests {
 
     #[test]
     fn will_fail_on_missing_field() {
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("int"), Hocon::Integer(5));
         let doc = Hocon::Hash(hm);
 
@@ -819,7 +833,7 @@ mod tests {
 
     #[test]
     fn will_not_fail_on_extra_field() {
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("int"), Hocon::Integer(5));
         hm.insert(String::from("float"), Hocon::Integer(6));
         hm.insert(String::from("extra"), Hocon::Integer(10));
@@ -831,14 +845,14 @@ mod tests {
 
     #[test]
     fn will_fail_on_wrong_type() {
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("int"), Hocon::Integer(5));
         hm.insert(String::from("float"), Hocon::String(String::from("wrong")));
         let doc = Hocon::Hash(hm);
         let res: super::Result<Simple> = dbg!(super::from_hocon(dbg!(doc)));
         assert!(res.is_err());
 
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("int"), Hocon::Integer(56));
         hm.insert(String::from("float"), Hocon::Real(543.12));
         hm.insert(String::from("boolean"), Hocon::Boolean(false));
@@ -848,7 +862,7 @@ mod tests {
         let res: super::Result<WithSubStruct> = dbg!(super::from_hocon(dbg!(doc)));
         assert!(res.is_err());
 
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("int"), Hocon::Integer(56));
         hm.insert(String::from("float"), Hocon::Real(543.12));
         hm.insert(String::from("boolean"), Hocon::Integer(1));
@@ -866,10 +880,10 @@ mod tests {
             a: Vec<i32>,
         }
 
-        let mut array = HashMap::new();
+        let mut array = LinkedHashMap::new();
         array.insert(String::from("0"), Hocon::Integer(5));
         array.insert(String::from("2"), Hocon::Integer(7));
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("a"), Hocon::Hash(array));
         let doc = Hocon::Hash(hm);
 
@@ -887,7 +901,7 @@ mod tests {
         }
 
         // let s: MyStruct = HoconLoader::new().load_str("").unwrap().resolve().unwrap();
-        let doc = Hocon::Hash(HashMap::new());
+        let doc = Hocon::Hash(LinkedHashMap::new());
 
         let res: super::Result<MyStructWithDefaultField> = dbg!(super::from_hocon(dbg!(doc)));
         assert!(res.is_ok());
@@ -904,7 +918,7 @@ mod tests {
             item: UnitStruct,
         }
 
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("item"), Hocon::Null);
         let doc = Hocon::Hash(hm);
 
@@ -920,7 +934,7 @@ mod tests {
             item: (u64, String),
         }
 
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         let mut vec_sub = vec![];
         vec_sub.push(Hocon::Integer(0));
         vec_sub.push(Hocon::String(String::from("Hello")));
@@ -942,7 +956,7 @@ mod tests {
             item: TupleStruct,
         }
 
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         let mut vec_sub = vec![];
         vec_sub.push(Hocon::Integer(0));
         vec_sub.push(Hocon::String(String::from("Hello")));
@@ -964,8 +978,8 @@ mod tests {
             item: HashMap<String, u64>,
         }
 
-        let mut hm = HashMap::new();
-        let mut hm_sub = HashMap::new();
+        let mut hm = LinkedHashMap::new();
+        let mut hm_sub = LinkedHashMap::new();
         hm_sub.insert(String::from("Hello"), Hocon::Integer(7));
         hm.insert(String::from("item"), Hocon::Hash(hm_sub));
         let doc = Hocon::Hash(hm);
@@ -983,7 +997,7 @@ mod tests {
             B,
         }
 
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("A"), Hocon::Integer(1));
         hm.insert(String::from("B"), Hocon::Integer(2));
         let doc = Hocon::Hash(hm);
@@ -997,8 +1011,8 @@ mod tests {
             s: u8,
         }
 
-        let mut hm = HashMap::new();
-        let mut hm_sub = HashMap::new();
+        let mut hm = LinkedHashMap::new();
+        let mut hm_sub = LinkedHashMap::new();
         hm_sub.insert(String::from("s"), Hocon::Integer(7));
         hm.insert(String::from("A"), Hocon::Hash(hm_sub));
         let doc = Hocon::Hash(hm);
@@ -1022,7 +1036,7 @@ mod tests {
 
     #[test]
     fn deserialize_unit_enum() {
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(
             String::from("item"),
             Hocon::String(String::from("UnitVariant")),
@@ -1036,11 +1050,11 @@ mod tests {
 
     #[test]
     fn deserialize_tuple_enum() {
-        let mut hm = HashMap::new();
-        let mut sub_hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
+        let mut sub_hm = LinkedHashMap::new();
         sub_hm.insert(String::from("u"), Hocon::Integer(12));
         sub_hm.insert(String::from("b"), Hocon::Boolean(true));
-        let mut variant_map = HashMap::new();
+        let mut variant_map = LinkedHashMap::new();
         variant_map.insert(String::from("StructVariant"), Hocon::Hash(sub_hm));
         hm.insert(String::from("item"), Hocon::Hash(variant_map));
         let doc = Hocon::Hash(hm);
@@ -1055,11 +1069,11 @@ mod tests {
 
     #[test]
     fn deserialize_struct_enum() {
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         let mut sub_vec = vec![];
         sub_vec.push(Hocon::Integer(7));
         sub_vec.push(Hocon::Boolean(false));
-        let mut variant_map = HashMap::new();
+        let mut variant_map = LinkedHashMap::new();
         variant_map.insert(String::from("TupleVariant"), Hocon::Array(sub_vec));
         hm.insert(String::from("item"), Hocon::Hash(variant_map));
         let doc = Hocon::Hash(hm);
@@ -1085,8 +1099,8 @@ mod tests {
             Asap { num_retries: u32 },
         }
 
-        let mut hm = HashMap::new();
-        let mut sub_hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
+        let mut sub_hm = LinkedHashMap::new();
         // sub_hm.insert(String::from("type"), Hocon::String(String::from("NoRetry")));
         sub_hm.insert(String::from("type"), Hocon::String(String::from("Asap")));
         sub_hm.insert(String::from("num_retries"), Hocon::Integer(7));
