@@ -1,4 +1,4 @@
-use std::collections::HashMap;
+use linked_hash_map::LinkedHashMap;
 use std::ops::Index;
 
 /// An HOCON document
@@ -76,7 +76,7 @@ pub enum Hocon {
     /// An array of `Hocon` values
     Array(Vec<Hocon>),
     /// An HashMap of `Hocon` values with keys
-    Hash(HashMap<String, Hocon>),
+    Hash(LinkedHashMap<String, Hocon>),
     /// A null value
     Null,
     /// A `BadValue`, marking an error in parsing or a missing value
@@ -196,7 +196,21 @@ macro_rules! units {
                 Some((value, $first_unit))
                 $(
                     | Some((value, $unit))
-                )* => Some(value * $scale),
+                )* => Some((value * $scale as f64) as u64),
+            )*
+            _ => None
+        }
+    };
+}
+
+macro_rules! units_f {
+    ( match $input:expr, $( $first_unit:expr, $( $unit:expr ),* => $scale:expr ),* ) => {
+        match $input {
+            $(
+                Some((value, $first_unit))
+                $(
+                    | Some((value, $unit))
+                )* => Some(value * $scale ),
             )*
             _ => None
         }
@@ -217,34 +231,34 @@ impl Hocon {
     /// # fn main() -> Result<(), Error> {
     /// assert_eq!(
     ///     HoconLoader::new().load_str(r#"{ size = 1.5KiB }"#)?.hocon()?["size"].as_bytes(),
-    ///     Some(1536.0)
+    ///     Some(1536)
     /// );
     /// # Ok(())
     /// # }
     /// ```
-    pub fn as_bytes(&self) -> Option<f64> {
+    pub fn as_bytes(&self) -> Option<u64> {
         match *self {
-            Hocon::Integer(ref i) => Some(*i as f64),
-            Hocon::Real(ref f) => Some(*f),
+            Hocon::Integer(ref i) => Some(*i as u64),
+            // Hocon::Real(ref f) => Some(*f),
             Hocon::String(ref s) => units!(
                 match unit_format::value_and_unit(s).map(|(value, unit)| (value, unit.trim())),
-                 "", "B", "b", "byte", "bytes"                     => 1.0,
-                 "kB", "kilobyte", "kilobytes"                     => 10.0f64.powf(3.0),
-                 "MB", "megabyte", "megabytes"                     => 10.0f64.powf(6.0),
-                 "GB", "gigabyte", "gigabytes"                     => 10.0f64.powf(9.0),
-                 "TB", "terabyte", "terabytes"                     => 10.0f64.powf(12.0),
-                 "PB", "petabyte", "petabytes"                     => 10.0f64.powf(15.0),
-                 "EB", "exabyte", "exabytes"                       => 10.0f64.powf(18.0),
-                 "ZB", "zettabyte", "zettabytes"                   => 10.0f64.powf(21.0),
-                 "YB", "yottabyte", "yottabytes"                   => 10.0f64.powf(24.0),
-                 "K", "k", "Ki", "KiB", "kibibyte", "kibibytes"    => 2.0f64.powf(10.0),
-                 "M", "m", "Mi", "MiB", "mebibyte", "mebibytes"    => 2.0f64.powf(20.0),
-                 "G", "g", "Gi", "GiB", "gibibyte", "gibibytes"    => 2.0f64.powf(30.0),
-                 "T", "t", "Ti", "TiB", "tebibyte", "tebibytes"    => 2.0f64.powf(40.0),
-                 "P", "p", "Pi", "PiB", "pebibyte", "pebibytes"    => 2.0f64.powf(50.0),
-                 "E", "e", "Ei", "EiB", "exbibyte", "exbibytes"    => 2.0f64.powf(60.0),
-                 "Z", "z", "Zi", "ZiB", "zebibyte", "zebibytes"    => 2.0f64.powf(70.0),
-                 "Y", "y", "Yi", "YiB", "yobibyte", "yobibytes"    => 2.0f64.powf(80.0)
+                 "", "B", "b", "byte", "bytes"                     => 1,
+                 "kB", "kilobyte", "kilobytes"                     => 10u64.pow(3),
+                 "MB", "megabyte", "megabytes"                     => 10u64.pow(6),
+                 "GB", "gigabyte", "gigabytes"                     => 10u64.pow(9),
+                 "TB", "terabyte", "terabytes"                     => 10u64.pow(12),
+                 "PB", "petabyte", "petabytes"                     => 10u64.pow(15),
+                 "EB", "exabyte", "exabytes"                       => 10u64.pow(18),
+                 "ZB", "zettabyte", "zettabytes"                   => 10u64.pow(21),
+                 "YB", "yottabyte", "yottabytes"                   => 10u64.pow(24),
+                 "K", "k", "Ki", "KiB", "kibibyte", "kibibytes"    => 2u64.pow(10),
+                 "M", "m", "Mi", "MiB", "mebibyte", "mebibytes"    => 2u64.pow(20),
+                 "G", "g", "Gi", "GiB", "gibibyte", "gibibytes"    => 2u64.pow(30),
+                 "T", "t", "Ti", "TiB", "tebibyte", "tebibytes"    => 2u64.pow(40),
+                 "P", "p", "Pi", "PiB", "pebibyte", "pebibytes"    => 2u64.pow(50),
+                 "E", "e", "Ei", "EiB", "exbibyte", "exbibytes"    => 2u64.pow(60),
+                 "Z", "z", "Zi", "ZiB", "zebibyte", "zebibytes"    => 2u64.pow(70),
+                 "Y", "y", "Yi", "YiB", "yobibyte", "yobibytes"    => 2u64.pow(80)
             ),
             _ => None,
         }
@@ -279,7 +293,7 @@ impl Hocon {
     }
 
     pub(crate) fn str_as_milliseconds(s: &str) -> Option<f64> {
-        units!(
+        units_f!(
             match unit_format::value_and_unit(s).map(|(value, unit)| (value, unit.trim())),
             "ns", "nano", "nanos", "nanosecond", "nanoseconds"          => 10.0f64.powf(-6.0),
             "us", "micro", "micros", "microsecond", "microseconds"      => 10.0f64.powf(-3.0),
@@ -663,7 +677,7 @@ mod tests {
 
     #[test]
     fn access_on_hash() {
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("a"), Hocon::Integer(5));
         hm.insert(String::from("b"), Hocon::Integer(6));
         let val = Hocon::Hash(hm);
@@ -695,7 +709,7 @@ mod tests {
 
     #[test]
     fn access_hash_as_array() {
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("0"), Hocon::Integer(5));
         hm.insert(String::from("a"), Hocon::Integer(6));
         hm.insert(String::from("2"), Hocon::Integer(7));
@@ -717,7 +731,6 @@ mod tests {
     fn access_on_bytes() {
         let val = Hocon::Array(vec![
             Hocon::Integer(5),
-            Hocon::Real(6.5),
             Hocon::String(String::from("7")),
             Hocon::String(String::from("8kB")),
             Hocon::String(String::from("9 EB")),
@@ -726,93 +739,95 @@ mod tests {
             Hocon::Boolean(false),
         ]);
 
-        assert_eq!(val[0].as_bytes(), Some(5.0));
-        assert_eq!(val[1].as_bytes(), Some(6.5));
-        assert_eq!(val[2].as_bytes(), Some(7.0));
-        assert_eq!(val[3].as_bytes(), Some(8.0 * 1_000.0));
-        assert_eq!(val[4].as_bytes(), Some(9.0 * 10.0f64.powf(18.0)));
-        assert_eq!(val[5].as_bytes(), Some(10.5 * 2.0f64.powf(20.0)));
+        assert_eq!(val[0].as_bytes(), Some(5));
+        assert_eq!(val[1].as_bytes(), Some(7));
+        assert_eq!(val[2].as_bytes(), Some(8 * 1_000));
+        assert_eq!(val[3].as_bytes(), Some(9 * 10u64.pow(18)));
+        // assert_eq!(val[4].as_bytes(), Some((10.5 * 2.0f64.powf(20.0)) as u64));
+        assert_eq!(val[4].as_bytes(), Some(11010048));
+        assert_eq!(val[5].as_bytes(), None);
         assert_eq!(val[6].as_bytes(), None);
-        assert_eq!(val[7].as_bytes(), None);
     }
 
     #[test]
     fn access_on_bytes_all_bytes_units() {
         for unit in vec!["B", "b", "byte", "bytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8));
         }
 
         for unit in vec!["kB", "kilobyte", "kilobytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 10.0f64.powf(3.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 10u64.pow(3)));
         }
         for unit in vec!["MB", "megabyte", "megabytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 10.0f64.powf(6.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 10u64.pow(6)));
         }
         for unit in vec!["GB", "gigabyte", "gigabytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 10.0f64.powf(9.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 10u64.pow(9)));
         }
         for unit in vec!["TB", "terabyte", "terabytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 10.0f64.powf(12.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 10u64.pow(12)));
         }
         for unit in vec!["PB", "petabyte", "petabytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 10.0f64.powf(15.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 10u64.pow(15)));
         }
         for unit in vec!["EB", "exabyte", "exabytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 10.0f64.powf(18.0)));
-        }
-        for unit in vec!["ZB", "zettabyte", "zettabytes"] {
-            let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 10.0f64.powf(21.0)));
-        }
-        for unit in vec!["YB", "yottabyte", "yottabytes"] {
-            let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 10.0f64.powf(24.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 10u64.pow(18)));
         }
+        // Overflow
+        // for unit in vec!["ZB", "zettabyte", "zettabytes"] {
+        //     let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
+        //     assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 10u64.pow(21)));
+        // }
+        // for unit in vec!["YB", "yottabyte", "yottabytes"] {
+        //     let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
+        //     assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 10u64.pow(24)));
+        // }
 
         for unit in vec!["K", "k", "Ki", "KiB", "kibibyte", "kibibytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 2.0f64.powf(10.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 2u64.pow(10)));
         }
         for unit in vec!["M", "m", "Mi", "MiB", "mebibyte", "mebibytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 2.0f64.powf(20.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 2u64.pow(20)));
         }
         for unit in vec!["G", "g", "Gi", "GiB", "gibibyte", "gibibytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 2.0f64.powf(30.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 2u64.pow(30)));
         }
         for unit in vec!["T", "t", "Ti", "TiB", "tebibyte", "tebibytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 2.0f64.powf(40.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 2u64.pow(40)));
         }
         for unit in vec!["P", "p", "Pi", "PiB", "pebibyte", "pebibytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 2.0f64.powf(50.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 2u64.pow(50)));
         }
         for unit in vec!["E", "e", "Ei", "EiB", "exbibyte", "exbibytes"] {
             let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 2.0f64.powf(60.0)));
-        }
-        for unit in vec!["Z", "z", "Zi", "ZiB", "zebibyte", "zebibytes"] {
-            let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 2.0f64.powf(70.0)));
-        }
-        for unit in vec!["Y", "y", "Yi", "YiB", "yobibyte", "yobibytes"] {
-            let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
-            assert_eq!(dbg!(val)[0].as_bytes(), Some(8.0 * 2.0f64.powf(80.0)));
+            assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 2u64.pow(60)));
         }
+        // overflow
+        // for unit in vec!["Z", "z", "Zi", "ZiB", "zebibyte", "zebibytes"] {
+        //     let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
+        //     assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 2u64.pow(70)));
+        // }
+        // for unit in vec!["Y", "y", "Yi", "YiB", "yobibyte", "yobibytes"] {
+        //     let val = Hocon::Array(vec![Hocon::String(format!("8{}", unit))]);
+        //     assert_eq!(dbg!(val)[0].as_bytes(), Some(8 * 2u64.pow(80)));
+        // }
     }
 
     #[test]
     fn access_on_duration() {
-        let mut hm = HashMap::new();
+        let mut hm = LinkedHashMap::new();
         hm.insert(String::from("ns"), Hocon::String(String::from("1ns")));
         hm.insert(String::from("us"), Hocon::String(String::from("1us")));
         hm.insert(String::from("ms"), Hocon::String(String::from("1ms")));
